# Changelog

All notable changes to the ReZEN Python client will be documented in this file.

The format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),
and this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).

## [Unreleased]

## [2.0.0] - 2025-06-08

### Added
- **Users API**: Complete implementation for user profile access, team membership, and office details
- **Owner Agent Support**: Enhanced Transaction Builder with owner agent functionality requiring proper transaction sequence
- **Keymaker Integration**: Access agent IDs through user keymaker endpoint for transaction operations
- **Convenience Methods**: `set_current_user_as_owner_agent()` for simplified owner agent setup
- **Enhanced Error Handling**: New exception types for better developer experience:
  - `InvalidFieldNameError`: Catches incorrect field names (e.g., `address` vs `street`)
  - `InvalidFieldValueError`: Validates field formats (e.g., uppercase state codes)
  - `TransactionSequenceError`: Identifies when operations are called in wrong order
- **Field Validation**: Pre-API validation for common field name and format errors

### Changed
- **Transaction Builder**: Owner agent endpoint now requires specific setup sequence (location → price/date → clients → owner agent)
- **API Coverage**: Total endpoints increased from 155 to 158 with the addition of Users API
- **Error Messages**: More descriptive error messages with specific field correction suggestions
- **Validation**: Methods now validate field names and values before making API calls

### Fixed
- **Owner Agent**: Discovered and documented the required transaction setup sequence for successful owner agent addition
- **Field Name Validation**: Methods now prevent common field name mistakes before API calls
- **Commission Endpoints**: Multipart/form-data handling for commission payer endpoint

## [1.5.0] - 2025-06-01

### Added
- **Docker Support**: Complete containerization for background agents and automated workflows
- **Background Agent Framework**: Template classes for creating continuous monitoring agents
- **Sample Agent**: Ready-to-use background agent template with logging and graceful shutdown
- **Transaction Monitor**: Advanced agent for monitoring transaction status changes with state persistence
- **Docker Compose Configuration**: Easy container orchestration with environment variable support
- **Cursor IDE Integration**: Configured environment.json for seamless Docker development in Cursor
- **Agent Templates**: Extensible base classes for custom background service development
- **Volume Mounting**: Live-update support for agent scripts and log file access
- **Security Features**: Non-root user containers with proper permission management
- **Comprehensive Documentation**: Docker setup guides for both general use and Cursor-specific workflows

### Infrastructure
- **Dockerfile**: Optimized Python 3.11 container with ReZEN client pre-installed
- **docker-compose.yml**: Production-ready orchestration with resource limits and health checks
- **.dockerignore**: Optimized build context for faster container builds
- **Environment Configuration**: Support for API keys, logging levels, and execution intervals
- **Log Management**: Persistent logging with host-accessible log files
- **Development Mode**: Separate dev container for testing and debugging

### Documentation
- **DOCKER_README.md**: Comprehensive Docker setup and usage guide
- **Agent Development Guide**: Instructions for creating custom background agents
- **Deployment Examples**: Production deployment patterns and best practices

## [1.1.4] - 2025-01-06

### Fixed
- **Client Base URL Handling**: Fixed RezenClient to properly handle custom base URLs for different API services. Teams, Agents, and Directory clients now correctly use their default yenta base URL instead of inheriting custom base URLs from the main client
- **Code Style**: Resolved all Black formatting inconsistencies and import sorting issues
- **Unused Imports**: Removed unused imports from transaction_builder.py and transactions.py modules
- **Line Length**: Fixed long lines in base_client.py for better code readability

### Technical Improvements
- All 266 tests passing with 100% code coverage maintained
- Improved code formatting consistency across the entire codebase
- Enhanced type safety with proper import organization

## [1.1.1] - 2024-01-21

### Fixed
- **Type Safety**: Resolved all mypy type checking errors for stricter type safety
- **Code Quality**: Applied black code formatting for consistent code style
- **Directory API**: Fixed `get_vendor_w9_url` method to handle both string and dict API responses
- **Method Signatures**: Removed problematic `post` method override that conflicted with base class signature

### Technical Improvements
- All 263 tests passing with 100% code coverage maintained
- Full mypy compliance with strict type checking enabled
- Consistent code formatting with black applied across entire codebase

## [1.1.0] - 2024-01-20

### Added
- **Directory API Client**: Complete implementation of Directory API with vendor and person management
- **Vendor Management**: Create, update, search, archive vendors with W9 file handling
- **Person Management**: Create, update, search, link/unlink persons to vendors
- **Directory Search**: Unified search across vendors and persons with advanced filtering
- **Role Management**: Get permitted roles for directory entries with type filtering
- **Geographic Support**: US states, Canadian provinces, and country enums for address data
- **Comprehensive Testing**: 41 new tests achieving 100% test coverage across all modules
- **DirectoryClient Export**: Added DirectoryClient and related enums to main package exports

### Changed
- **Enhanced API Coverage**: Now covers Transaction Builder, Transactions, Teams, Agents, and Directory APIs
- **Improved Package Description**: Updated to reflect all supported API modules
- **Better Test Coverage**: Achieved 100% test coverage across entire codebase (1,004 statements)

### Fixed
- **Missing Test Coverage**: Added tests for previously uncovered code paths in TeamsClient and RezenClient
- **Type Safety**: Improved type hints and error handling in all client modules
- **API Endpoint URLs**: Corrected directory API endpoint paths for proper functionality

## [1.0.7] - 2024-01-15

### Added
- Complete API wrapper implementation for all ReZEN endpoints
- Transaction Builder API with 60+ endpoints
- Transactions API with 50+ endpoints
- Agents API with 35+ endpoints
- Teams API with 10+ endpoints
- Comprehensive error handling with custom exceptions
- 100% test coverage with pytest
- Type hints throughout the codebase
- Google-style docstrings for all public methods

### Security
- Secure API key handling with environment variable support

## [1.0.6] - 2024-01-10

### Added
- Teams API client with search and management capabilities
- TeamStatus, TeamType, SortDirection, and SortField enums
- Advanced filtering for team searches
- Pagination support for large result sets

### Fixed
- Date handling in team search filters
- Enum value validation for team parameters

## [1.0.5] - 2024-01-05

### Added
- Agents API client with comprehensive search capabilities
- Agent network hierarchy management
- AgentStatus, AgentSortDirection, AgentSortField enums
- Country and StateOrProvince enums for geographic filtering
- Tax forms and payment details endpoints
- Network size analysis by tier

### Changed
- Improved agent search with better filtering options
- Enhanced pagination handling for large agent datasets

## [1.0.4] - 2024-01-01

### Added
- Transaction Builder API with complete CRUD operations
- Support for all transaction participant types
- Commission and financial management endpoints
- Property and location information management
- File upload capabilities for documents

### Fixed
- Transaction ID validation in API calls
- Proper handling of multipart form data uploads

## [1.0.3] - 2023-12-28

### Added
- Transactions API for live transaction management
- Participant management for existing transactions
- Financial operations support
- Document generation and retrieval

### Changed
- Improved base client with better session management
- Enhanced error handling with retry mechanisms

## [1.0.2] - 2023-12-25

### Added
- Base client architecture with HTTP session management
- Custom exception hierarchy for different error types
- Authentication error handling
- Rate limiting support with backoff strategies

### Fixed
- SSL certificate verification issues
- Timeout handling for slow API responses

## [1.0.1] - 2023-12-20

### Added
- Initial project structure
- Basic client configuration
- Environment variable support for API keys

### Fixed
- Package installation issues
- Import path corrections

## [1.0.0] - 2023-12-15

### Added
- Initial release of ReZEN Python client
- Basic API client framework
- Project setup and configuration

---

## Release Notes

### Version 1.0.7 - Complete API Coverage

This major release represents the completion of comprehensive API coverage for all ReZEN endpoints. Key highlights:

**🎉 Complete API Coverage**
- 150+ endpoints fully implemented and tested
- Transaction Builder: 60+ endpoints for complete transaction lifecycle
- Transactions: 50+ endpoints for live transaction management
- Agents: 35+ endpoints for agent discovery and network management
- Teams: 10+ endpoints for team operations

**🔒 Production Ready**
- 100% test coverage with over 500 unit and integration tests
- Comprehensive error handling with custom exception hierarchy
- Type hints throughout for excellent IDE support
- Google-style docstrings for all public methods

**📚 Documentation Excellence**
- Complete API reference with examples for every endpoint
- Real-world usage patterns and best practices
- Troubleshooting guide for common issues
- Contributing guide for developers

**🚀 Developer Experience**
- Intuitive client interface with logical method organization
- Enum support for better IDE autocomplete and validation
- Flexible authentication options (env vars, direct, .env files)
- Comprehensive examples for common workflows

### Version 1.0.6 - Teams Management

Introduced comprehensive team management capabilities:

- **Team Discovery**: Search teams by name, type, status, and creation date
- **Advanced Filtering**: Support for all team types (Normal, Platinum, Group, etc.)
- **Pagination**: Efficient handling of large team datasets
- **Type Safety**: Complete enum support for all team parameters

### Version 1.0.5 - Agent Networking

Added powerful agent network management:

- **Agent Search**: Find agents by name, location, status with advanced filtering
- **Network Analysis**: Explore agent sponsor trees and downline hierarchies
- **Geographic Filtering**: Search by country, state, or province
- **Financial Integration**: Access tax forms, payment details, and commission plans

### Version 1.0.4 - Transaction Building

Comprehensive transaction builder implementation:

- **Transaction Lifecycle**: Complete support from creation to submission
- **Participant Management**: Add buyers, sellers, agents, and service providers
- **Financial Operations**: Commission splits, fee management, payment processing
- **Document Handling**: File uploads for contracts and supporting documents

### Version 1.0.3 - Transaction Management

Live transaction management capabilities:

- **Transaction Operations**: Full CRUD operations for existing transactions
- **Participant Updates**: Modify participant information and roles
- **Financial Tracking**: Payment processing and financial reconciliation
- **Reporting**: Generate transaction summaries and documentation

---

## Upgrade Guide

### Upgrading to 1.0.7

This release is fully backward compatible. Simply upgrade:

```bash
pip install --upgrade rezen
```

**New Features Available:**
- Enhanced documentation with MkDocs
- Additional utility methods for common operations
- Improved error messages with more context

### Upgrading from 1.0.6 to 1.0.7

No breaking changes. All existing code will continue to work.

### Upgrading from 1.0.5 to 1.0.6

No breaking changes. Team API additions are purely additive.

### Upgrading from Earlier Versions

If upgrading from versions prior to 1.0.5, review the API changes in each release:

1. **Agent API Changes (1.0.5)**: New enum values and search parameters
2. **Transaction API Changes (1.0.4)**: Enhanced participant management
3. **Error Handling Changes (1.0.2)**: New exception hierarchy

---

## Contributing

Found a bug or want to contribute? See our [Contributing Guide](../development/contributing.md) for details on:

- Setting up the development environment
- Running tests and ensuring quality
- Submitting pull requests
- Reporting issues

## Support

- **Documentation**: [Full Documentation](../index.md)
- **GitHub Issues**: [Report Issues](https://github.com/theperrygroup/rezen/issues)
- **Email**: [support@rezen.com](mailto:support@rezen.com)
